@@ -24,7 +24,7 @@ our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 
 our @EXPORT = qw( );
 
-our $VERSION = '0.01';
+our $VERSION = '0.03';
 
 sub new {
 
@@ -38,7 +38,7 @@ sub new {
         $self->{path} = $params{path};
     }
     else {
-        $self->{path} = "~/.hstore";
+        $self->{path} = "~/.ustore";
     }
 
     if ( exists($params{prefix}) ) {
@@ -90,7 +90,7 @@ sub add {
     my $destStoredFile = File::Spec->catfile($SSubDir , $self->{prefix}.$uuidString);
 
         copy( $file, $destStoredFile )
-        or die "Unable to copy file into hstore as $destStoredFile";
+        or die "Unable to copy file into ustore as $destStoredFile";
 
     return $uuidString;
 }
@@ -117,8 +117,6 @@ sub remove {
 
             unlink($destStoredFile) or return undef;
 
-        #die "Unable to delete file from hstore named $destStoredFile";
-        #return undef;
     }
     else {
         return;
@@ -126,6 +124,30 @@ sub remove {
 
 }
 
+sub get {
+
+    my ( $self, $id ) = @_;
+
+    my $destStoredFile;
+
+    my $SSubDir;
+    my @tempstr = split (//,$id);
+    my @dirtree;
+    for (1 .. $self->{depth}){
+      push @dirtree , shift @tempstr;    
+    }
+    $SSubDir = File::Spec->catdir($self->{path} , @dirtree);
+
+    $destStoredFile = File::Spec->catfile($SSubDir , $self->{prefix}.$id);
+
+    if ( -e $destStoredFile ) {
+        open ( my $fh, '<', $destStoredFile );
+	return *$fh;
+    } else {
+	return;
+    }
+}
+
 sub getpath {
 
     my ( $self, $id ) = @_;
@@ -175,6 +197,10 @@ File::UStore - Perl extension to store files  on a filesystem using a non-hash U
   # Add a file in the store
   my $id = $store->add(*$FH);
 
+  # Get file handle from id. this might not work if you have too wierd a storage scheme.
+  my $handle = $store->get($id);
+  print <$handle>;
+
   # Return the filesystem location of an id
   my $location = $store->getpath($id);
 
@@ -201,8 +227,8 @@ everytime a file is to be stored.
 
 The Module also provides a option to choose the folder depth at which a file is stored.
 This can be changed from the default value of 3. Increasing depth is advisable 
-if the Store might contain a large number of files in your use case. This helps
-avoid having an excessive number of files in any single folder.
+if the store might contain a large number of files in your use case. This will help
+to avoid having a too large number of files in any single folder.
 
 =head1 METHODS
 
@@ -215,14 +241,11 @@ The following methods are provided:
 
 =item $store = File::UStore->new( path => "/home/shantanu/.teststore", prefix => "prefix_", depth  => 5 );
 
-This constructor  returns a new C<File::HFile>  object encapsulating a
+This constructor  returns a new C<File::UStore>  object encapsulating a
 specific store. The path specifies  where the UStore is located on the
-filesystem.  If the  path  is  not specified,  the  path ~/.hstore  is
-used. The digest specifies the algorithm to be used (SHA-1 or SHA-2 or
-the  submission   date  called  FAT).  If  not   specified,  SHA-1  is
-used. Various digest can be mixed  in the same path but the utility is
-somewhat limited.  The $prefix is only  an extension used  for the FAT
-(Free Archive Format) format to specify the archive unique name.
+filesystem.  If the  path  is  not specified,  the  path ~/.ustore  is
+used. The $prefix is an extension to specify the prefix appended before 
+unique file name.
 
 =item $store->add($filename)
 
@@ -231,6 +254,12 @@ is the id ($id) of the $filename stored. From this point on the user
 will only be able to refer to this file using the id.
 Return undef on error. 
 
+=item $store->get($id)
+
+Return the file handle of the file from its id.
+
+Return undef on error.
+
 =item $store->getpath($id)
 
 Return the filesystem location of the file from its id.
@@ -255,9 +284,37 @@ http://www.usenix.org/events/hotos03/tech/full_papers/henson/henson.pdf
 
 Shantanu Bhadoria, E<lt>shantanu (dot comes here) bhadoria at gmail dot comE<gt>
 
+=head1 USE CASE FOR THIS MODULE IN LIEU OF A HASH BASED STORAGE
+
+File::HStore by Alexandre Dulaunoy, is a similar module that
+provides File Hash based storage. However due to the nature of File
+Hashing, File::HStore doesn't allow duplicates. If the same file is
+stored a second time using File::HStore it transparently returns the
+same hash it had returned last time as an id without adding any new 
+file in storage due to inherent character of hash based storage, while 
+this is useful if a user doesn't want any duplicates occurring in a
+storage, this apparently trivial difference is risky in the use case
+where two processes upload a duplicate file to the store and both
+processes want to do file handling simultaneously, only one of the 
+processes will be able to get a lock(deletion,manipulation etc.) on 
+the file at a time and if the first process deletes the file referred 
+to by its ID, the second process will never know what happened to the 
+file it added. 
+
+Hence to serve this unique use case I wrote this module for a UUID 
+based storage solution which is not hostage to auto de-duping features 
+of HStore. This module returns a unique file id each time a file is 
+uploaded even if its a duplicate of existing file previously uploaded. 
+This allows multiple processes handling data from a common file dump to 
+access the same file. This module also expands on the Hstore to allow 
+the user flexibility of choosing the depth of storage to optimize the
+performance for the users application. Depth of storage allows users
+to make the balance between average "number of files in a folder"
+and folder depth.
+
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2011 by Shantanu Bhadoria  <shantanu (dot comes here) bhadoria at gmail dot com>
+Copyright (C) 2011 by Shantanu Bhadoria  <shantanu (dot comes here) bhadoria at gmail dot com> http://www.shantanubhadoria.com
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.5 or,
